// Copyright (c) 2020 NetApp, Inc. All rights reserved.

package deepcopy

import (
	"encoding/json"
	"fmt"
	"reflect"

	"github.com/thoas/go-funk"
)

// Copy makes a deep copy from src into dst
// panics if json marshall/unmarshal fails
// do not use if the type changes dynamically base on runtime
// Note: we use this in code with well defined struct and json tags,
// so none of the panics should occur
func Copy(dst interface{}, src interface{}) {
	if src == nil {
		panic("src cannot be nil")
	}

	if dst == nil {
		panic("dst cannot be nil")
	}

	bytes, err := json.Marshal(src)
	if err != nil {
		panic(fmt.Sprintf("Failed to marshal src: %s", err))
	}

	err = json.Unmarshal(bytes, dst)
	if err != nil {
		panic(fmt.Sprintf("Failed to unmarshal into dst: %s", err))
	}
}

// CopyStructSlice convert slice of one struct to another struct using
// json marshal and unmarshal methods.
// inputSlice should be a slice of structs and resultSlicePtr should be a
// pointer to a slice of struct.
// Deprecated, use Copy instead, which works on slice directly
func CopyStructSlice(inputSlice interface{}, resultSlicePtr interface{}) {
	if reflect.TypeOf(inputSlice).Kind() != reflect.Slice {
		panic("First parameter must be an iterable")
	}

	// we must use ptr to slice to modify it.
	if reflect.TypeOf(resultSlicePtr).Kind() != reflect.Ptr {
		panic("Second parameter must be an pointer to iterable")
	}
	resultSlicePtrValue := reflect.ValueOf(resultSlicePtr)
	resultSliceValue := resultSlicePtrValue.Elem()
	if !funk.IsIteratee(resultSliceValue.Interface()) {
		panic("Second parameter must be an pointer to iteratee")
	}

	// get type of result
	resultType := resultSliceValue.Type().Elem()

	_ = funk.Map(inputSlice, func(doc interface{}) interface{} {
		jArr, err := json.Marshal(doc)
		if err != nil {
			panic(err)
		}

		// create model struct and initialize to zero value of that type
		var resultStructPtr interface{}
		if resultType.Kind() == reflect.Ptr {
			// allocate a pointer to model.
			// New(type) gives pointer to type
			resultStructPtr = reflect.New(resultType.Elem()).Interface()
		} else {
			// since model type is not pointer, New(type) gives pointer
			resultStructPtr = reflect.New(resultType).Interface()
		}

		err = json.Unmarshal(jArr, &resultStructPtr)
		if err != nil {
			panic(err)
		}

		// append the new element with appropriate type
		var newEle reflect.Value
		if resultType.Kind() == reflect.Ptr {
			newEle = reflect.ValueOf(resultStructPtr)
		} else {
			newEle = reflect.ValueOf(resultStructPtr).Elem()
		}
		// append, and modify the slice
		resultSliceValue.Set(reflect.Append(resultSliceValue, newEle))

		return nil
	})

}
