// Copyright (c) 2020 NetApp, Inc. All rights reserved.

package log

import (
	"database/sql/driver"
	"fmt"
	"os"
	"path"
	"regexp"
	"runtime"
	"time"

	"github.com/sirupsen/logrus"
)

type UTCFormatter struct {
	logrus.Formatter
}

const (
	// PanicLevel level, highest level of severity. Logs and then calls panic with the
	// message passed to Debug, Info, ...
	PanicLevel logrus.Level = iota
	// FatalLevel level. Logs and then calls `os.Exit(1)`. It will exit even if the
	// logging level is set to Panic.
	FatalLevel
	// ErrorLevel level. Logs. Used for errors that should definitely be noted.
	// Commonly used for hooks to send errors to an error tracking service.
	ErrorLevel
	// WarnLevel level. Non-critical entries that deserve eyes.
	WarnLevel
	// InfoLevel level. General operational entries about what's going on inside the
	// application.
	InfoLevel
	// DebugLevel level. Usually only enabled when debugging. Very verbose logging.
	DebugLevel
)

const logLevelEnvName = "LOG_LEVEL"
const defaultLogLevel = "info"

// below code to mask the password field in logs is from netapp-client lib
// PasswordMask defines the mask used when logging out a password
const PasswordMask = "****************"

// Secret defines a type that outputs the password mask when called with String()
type Secret string
type Fields logrus.Fields

func (s Secret) String() string {
	return PasswordMask
}

func (s Secret) Value() (driver.Value, error) {
	return string(s), nil
}

func (s *Secret) Scan(src interface{}) error {
	if src == nil {
		*s = ""
	} else {
		*s = Secret(string(src.([]uint8)))
	}
	return nil
}

// WithFields creates an entry from the standard Logger and adds multiple
// fields to it. This is simply a helper for `WithField`, invoking it
// once for each field.
func WithFields(fields Fields) *logrus.Entry {
	return defaultLogger.WithFields(logrus.Fields(fields))
}

type sanitation struct {
	pattern     *regexp.Regexp
	replacement string
}

var (
	sanitationObj = sanitation{pattern: regexp.MustCompile(`(\\?"[^\\"]*[Pp]assword\\?":[^\\"]*)(\\?")[^\\"]*(\\?")|([Pp]assword=)[^ ]+`), replacement: fmt.Sprintf(`$4$1$2%s$3`, PasswordMask)}
)

var defaultLogger *logrus.Logger = newLogger()

// GetLogger returns the default global logger
func GetLogger() *logrus.Logger {
	return defaultLogger
}

func sanitize(formatted string) string {
	if sanitationObj.pattern.MatchString(formatted) {
		formatted = sanitationObj.pattern.ReplaceAllString(formatted, sanitationObj.replacement)
	}
	return formatted
}

func (u *UTCFormatter) Format(e *logrus.Entry) ([]byte, error) {
	e.Time = e.Time.UTC()
	e.Message = sanitize(e.Message)
	return u.Formatter.Format(e)
}

// getLogLevelFromEnv Returns the logrus log level constant as per the LOG_LEVEL env
// variable or defaults to InfoLevel
// require that log level be strings ex."debug"
func getLogLevelFromEnv() logrus.Level {
	value := os.Getenv("LOG_LEVEL")
	logrus.Debug("LOG_LEVEL value: ", value)
	if len(value) == 0 {
		logrus.Debugf("No LOG_LEVEL env variable found. Using default value: %v", defaultLogLevel)
		value = defaultLogLevel
	}

	// Get the logrus log level constant
	res, err := logrus.ParseLevel(value)
	if err != nil {
		// If the wrong string is provided as LOG_LEVEL env variable
		// then use the default log level
		res, _ := logrus.ParseLevel(defaultLogLevel)
		return res
	}
	return res
}

func newDefaultLoggerFormatter() logrus.Formatter {

	// Format the logging message so that:
	// It contain function name with ()
	// and use base filename instead of absolute path
	prettyfyFunc := func(f *runtime.Frame) (string, string) {
		filename := path.Base(f.File)
		return fmt.Sprintf("%s()", f.Function), fmt.Sprintf("%s:%d", filename, f.Line)
	}

	return &UTCFormatter{
		&logrus.TextFormatter{
			FullTimestamp:    true,
			TimestampFormat:  time.RFC3339Nano,
			CallerPrettyfier: prettyfyFunc,
		},
	}
}

/* return a new logger with custom settings .
Log Levels
 0 - Panic
 1 - Fatal
 2 - Error
 3 - Warning
 4 - Info  -- default value
 5 - Debug
*/
func newLogger() *logrus.Logger {
	customLogLevel := getLogLevelFromEnv()
	logrus.Debugf("Creating global logger with level %v", customLogLevel.String())
	return &logrus.Logger{
		Out:          os.Stdout,
		Level:        customLogLevel,
		Formatter:    newDefaultLoggerFormatter(),
		ReportCaller: true,
	}
}
