package ntaphttp

import (
	"crypto/tls"
	"net/http"
	"time"

	"github.com/PuerkitoBio/rehttp"
	"github.com/sirupsen/logrus"
	"github.com/thoas/go-funk"
)

// logRetry is a retry with logging and always returns true
// This is used to compose the retry, and log retry info
func logRetry(logger *logrus.Logger) rehttp.RetryFn {
	return func(attempt rehttp.Attempt) bool {
		if logger == nil { // skip logging if no logger
			return true
		}

		logFields :=
			logger.WithFields(logrus.Fields{
				"Attempt":       attempt.Index,
				"Attempt-Error": attempt.Error,
			})

		req := attempt.Request
		if req != nil {
			logFields = logFields.WithFields(logrus.Fields{
				"URL":    req.URL,
				"Method": req.Method,
			})
		}

		resp := attempt.Response
		if resp != nil {
			logFields = logFields.WithFields(logrus.Fields{
				"Status": resp.Status,
			})
			// TODO: one can add body here
		}

		if attempt.Index == 0 {
			// this is the first request
			logFields.Info("Rehttp Made Request")
		} else {
			logFields.Info("Rehttp Retried Request")
		}
		return true
	}
}

// NewHTTPRetryClient constructs and returns a new
// http.Client with retry in transport layer which is used in default NewOntapClient function
// Using config.
// NOTE: this retry http client can be plugged in all swagger generated client
func NewHTTPRetryClient(cfg *Config) *http.Client {
	//Use default logger if config is not set with any logger
	if cfg.Logger == nil {
		cfg = cfg.WithLogger(logrus.New())
	}

	// create the transport to disable SSL verification
	// this transport implements RoundTripper interface
	httprt := &http.Transport{TLSClientConfig: &tls.Config{
		InsecureSkipVerify: cfg.InsecureSkipVerify,
		// enforce tls version 1.2 and above
		MinVersion: tls.VersionTLS12,
	},
	// HTTP2 future support
	// ForceAttemptHTTP2: true,
	}

	// configure delay function
	delayFn := rehttp.ConstDelay(time.Duration(cfg.RetryDelayBaseSec) * time.Second)
	if cfg.RetryDelayType == ExponentialDelay {
		delayFn = rehttp.ExpJitterDelay(time.Duration(cfg.RetryDelayBaseSec)*time.Second,
			time.Duration(cfg.RequestTimeoutSec))
	}

	// Use rehttp as the underlying instead of http for retry
	tr := rehttp.NewTransport(
		httprt, // use constructed roundtripper
		rehttp.RetryAll(
			logRetry(cfg.Logger),
			// Max retry attempts for a single request
			rehttp.RetryMaxRetries(cfg.RetryMaxTrial),
			rehttp.RetryHTTPMethods(
				funk.Map(cfg.RetryHTTPMethods, func(x HTTPMethod) string {
					return string(x)
				}).([]string)...),
			rehttp.RetryAny(
				// include retriable http code here
				rehttp.RetryStatuses(cfg.RetryHTTPCodes...),
				// retry all temporary errors, transport timeout, etc.
				// This is not well documented in Golang Transport package
				// TODO: test this case
				rehttp.RetryTemporaryErr(),
			),
		),
		delayFn,
	)

	httpclient := &http.Client{
		Transport: tr,
		Timeout:   time.Duration(cfg.RequestTimeoutSec) * time.Second, // for each request the max timeout
	}
	return httpclient
}
