package taskqueue

import "time"

// TaskFunc is called when a task is time to be executed.
// In:
//  x is the task the this function should be performed
//    x.Content is supposed to be specific to the task
//    x is not a pointer to prevent modification of UUID and RunAt time that will break the heap state
//    TaskFunc is can be set in each task and in the TQueue default.
type TaskFunc func(x Task) ([]*Task, []string)

// Task represents a task in queue.
type Task struct {
	// UUID should be unique in TQueue
	UUID string
	// Time that the Task should be run
	RunAt time.Time
	// If true TQueue will skip executing this Task
	Cancelled bool
	// Any Task content that TaskFunc should perform on.
	Content interface{}
	// Func does the task, and is executed which this Task struct as parameter
	// If Func is nil, the default Func in TQueue is used instead.
	Func TaskFunc
	// optional: name of the task
	Name string
	// optional: comment for what this task does
	Comment string
}

// NewTask returns a new Task
func NewTask() *Task {
	return &Task{}
}

// WithUUID sets the UUID
func (t *Task) WithUUID(uuid string) *Task {
	t.UUID = uuid
	return t
}

// WithRunAt sets the RunAt
func (t *Task) WithRunAt(x time.Time) *Task {
	t.RunAt = x
	return t
}

// WithContent sets the Content
func (t *Task) WithContent(c interface{}) *Task {
	t.Content = c
	return t
}

// WithFunc sets the Func
func (t *Task) WithFunc(f TaskFunc) *Task {
	t.Func = f
	return t
}

// WithName sets the Name
func (t *Task) WithName(name string) *Task {
	t.Name = name
	return t
}

// WithComment sets the Comment
func (t *Task) WithComment(x string) *Task {
	t.Comment = x
	return t
}
