package taskqueue

import (
	"container/heap"
	"errors"
	"fmt"
	"sync"
	"time"
)

// --- Internal queue ---

// tQueue is a internal queue based on runAt time
type tQueue []*Task

func (t *tQueue) Push(x interface{}) {
	i := x.(*Task)
	// add to internals
	*t = append(*t, i)
}

// remove task at Len - 1
func (t *tQueue) Pop() interface{} {
	old := *t
	length := len(old)
	i := old[length-1]

	// remove the element from slice
	old[length-1] = nil
	*t = old[:length-1]

	return i
}

// Sort interface Len Less and Swap

func (t *tQueue) Len() int {
	return len(*t)
}

func (t tQueue) Less(i, j int) bool {
	// i time is before j time so i should be sorted before j
	return t[i].RunAt.Before(t[j].RunAt)
}

func (t tQueue) Swap(i, j int) {
	t[i], t[j] = t[j], t[i]
}

// --- Internal queue end ---

// Logger is used in this queue to log events
type Logger interface {
	Info(args ...interface{})
	Error(args ...interface{})
}

// TQueue is the task queue to hold tasks and execute them at their RunAt time.
type TQueue struct {
	// internal queue
	q tQueue
	// map from task UUID to task. For fast lookup and modify of the task
	m map[string]*Task
	// lock for the entire queue
	mtx sync.Mutex
	// default function to process task content. Task specified Func has higher priority than this.
	defaultFunc TaskFunc
	// logs events that is happening on this queue
	logger Logger
}

// NewTQueue returns a new TQueue
func NewTQueue() *TQueue {
	return &TQueue{
		q:   make(tQueue, 0),
		m:   make(map[string]*Task),
		mtx: sync.Mutex{},
	}
}

// Len returns the length of the queue
// including cancelled tasks
func (t *TQueue) Len() int {
	t.mtx.Lock()
	defer t.mtx.Unlock()

	return t.q.Len()
}

// Push adds a new task to queue
// x is non-ptr to prevent client modify uuid and runAt time that will break queue property
// If the task UUID already exists in the queue, cancel/skip the old one and add x.
func (t *TQueue) Push(x Task) {
	t.mtx.Lock()
	defer t.mtx.Unlock()
	t.push(x)
}

// push adds a new task to queue, without locking.
func (t *TQueue) push(x Task) {
	t.logInfo(fmt.Sprintf("Push Task %v", x.UUID))

	if old, ok := t.m[x.UUID]; ok {
		t.logInfo(fmt.Sprintf("Task %+v already in queue, cancel the old task, add the new task %+v", old, x))
		old.Cancelled = true
		// TODO: drop this task. Change priority should go through heap lib
		// old.RunAt = time.Now()
	}

	heap.Push(&t.q, &x)
	t.m[x.UUID] = &x
}

// popAtomic returns the latest task with locking
func (t *TQueue) popAtomic() *Task {
	t.mtx.Lock()
	defer t.mtx.Unlock()

	return t.pop()
}

// pop returns the next task from queue, without locking
func (t *TQueue) pop() *Task {
	x := heap.Pop(&t.q).(*Task)
	delete(t.m, x.UUID)
	return x
}

// Cancel cancels the task in the Queue
// it does not remove the task right away but when it is time to run the task it will be skipped
func (t *TQueue) Cancel(uuid string) error {
	t.mtx.Lock()
	defer t.mtx.Unlock()

	return t.cancel(uuid)
}

// cancel mark task with uuid as cancelled. Return error if not found
func (t *TQueue) cancel(uuid string) error {
	x, ok := t.m[uuid]
	if !ok {
		err := errors.New(fmt.Sprintf("task with uuid %v not found", uuid))
		t.logError(err)
		return err
	}
	x.Cancelled = true
	return nil
}

// PeekTask peeks the Task with uuid
// if not found return error.
// return Task to prevent client from modifying
func (t *TQueue) PeekTask(uuid string) (Task, error) {
	t.mtx.Lock()
	defer t.mtx.Unlock()

	x, ok := t.m[uuid]
	if !ok {
		err := errors.New(fmt.Sprintf("task with uuid %v not found", uuid))
		t.logError(err)
		return Task{}, err
	}
	return *x, nil
}

// Peek returns the next task to be popped, with locking
// panics if no task
func (t *TQueue) Peek() Task {
	t.mtx.Lock()
	defer t.mtx.Unlock()

	return t.peek()
}

// peek returns the next task to be popped
// panics if no task
func (t *TQueue) peek() Task {

	if len(t.q) == 0 {
		panic("Peek must be called when queue is not empty")
	}

	// top task is at index 0
	// see https://github.com/golang/go/issues/17510
	top := t.q[0]
	return *top
}

// SetDefaultFunc sets the default processing function for this queue
// The task's Func take priority if specified
func (t *TQueue) SetDefaultFunc(f TaskFunc) {
	t.defaultFunc = f
}

// SetLogger sets the logger of the queue, and will be used to log events
func (t *TQueue) SetLogger(logger Logger) {
	t.logger = logger
}

// logInfo logs the info using internal logger
func (t *TQueue) logInfo(args ...interface{}) {
	if t.logger == nil {
		return
	}
	t.logger.Info(args...)
}

// logError logs the error using internal logger
func (t *TQueue) logError(args ...interface{}) {
	if t.logger == nil {
		return
	}
	t.logger.Error(args...)
}

// Run runs all tasks in queue at their RunAt time, and sleeps while waiting for
// more tasks to be added.
// Intended Use:
// c := make(chan struct{})
// go t.Run(c)
// defer func(){
//   c <- struct{}{}
// }()
func (t *TQueue) Run(done <-chan struct{}) {

	// pop task from queue until the task is in future.
	// skip cancelled tasks.
	// sleep 1 sec if no task.
	// We do not sleep until the next task comes because there can be more immediate task
	// newly pushed. And the done signal needs to be quickly handled.
	for {

		sleepTime := 1 * time.Second

		if t.q.Len() > 0 {
			processed := t.runTaskAtomic()
			if processed {
				// if one task is processed, it means there might be another task waiting
				// so we do not sleep
				sleepTime = 0
			}
		}

		// stop or sleep
		select {
		case <-done:
			t.logInfo("Shutting Down TQueue")
			return
		default:
			time.Sleep(sleepTime)
		}

	}
}

// runTaskAtomic processes the first task on the queue
// do nothing if there is no task
// return:
//  true if one task is processed
//  false if no task is processed
func (t *TQueue) runTaskAtomic() bool {
	t.mtx.Lock()
	defer t.mtx.Unlock()

	// peek and see if we have a task ready
	if len(t.q) == 0 {
		return false
	}

	// there is some task so peek the first one
	p := t.q[0]

	timeNow := time.Now()
	if p.RunAt.After(timeNow) {
		// task is for future
		return false
	}

	// consume the task. Note: pop is not locked
	i := t.pop()

	if i.Cancelled {
		// skip cancelled tasks
		t.logInfo(fmt.Sprintf("Task cancelled %+v . skip.", i))
		return true
	}

	// do the task and add more tasks back to q
	// use the task specified Func if present, else use default
	funcToCall := t.defaultFunc
	if i.Func != nil {
		funcToCall = i.Func
	}
	if funcToCall == nil {
		panic("TQueue Default func is not set and task func is not set")
	}
	moreTasks, cancelUUIDs := funcToCall(*i)

	// Note: render content might break with non-data class task content
	t.logInfo(fmt.Sprintf("Task proccessed %v, with new tasks %+v, and cancel tasks %v",
		i.UUID, moreTasks, cancelUUIDs))

	for _, newTask := range moreTasks {
		t.push(*newTask)
	}

	for _, uuid := range cancelUUIDs {
		// cancel each and log error
		if err := t.cancel(uuid); err != nil {
			t.logError(err)
		}
	}
	return true
}
