// Copyright (c) 2020 NetApp, Inc. All rights reserved.

package wraphttp

import "net/http"

// WrapResponseWriter is a wrapper around an http.ResponseWriter which captures the
// status code written to the response, so that it can be logged.
type WrapResponseWriter struct {
	wrapped        http.ResponseWriter
	written        bool // if response is recorded yet
	WrapStatusCode int
	WrapBody       []byte
	wrapHeader     http.Header
}

// NewLoggingResponseWriter returns a LoggingResponseWriter which is a wrapper
// around http.ResponseWriter
func NewLoggingResponseWriter(wrapped http.ResponseWriter) *WrapResponseWriter {
	return &WrapResponseWriter{
		wrapped:        wrapped,
		written:        false,
		WrapStatusCode: 0,
		WrapBody:       nil,
		wrapHeader:     nil,
	}
}

// Implements ResponseWriter Interface

// Header returns the wrapped header
// if no wrapped writter, use the own header field
func (lrw *WrapResponseWriter) Header() http.Header {
	if lrw.wrapped == nil {
		if lrw.wrapHeader == nil {
			lrw.wrapHeader = http.Header{}
		}
		return lrw.wrapHeader
	}
	return lrw.wrapped.Header()
}

// Write call the write method of http's ResponseWriter
func (lrw *WrapResponseWriter) Write(content []byte) (int, error) {
	// TODO: here we want to type assert if it is wrapped by another wrapper.
	lrw.written = true
	lrw.WrapBody = append(lrw.WrapBody, content...)
	if lrw.wrapped == nil {
		return len(lrw.WrapBody), nil
	}
	return lrw.wrapped.Write(content)
}

// WriteHeader write the status code in the header
func (lrw *WrapResponseWriter) WriteHeader(statusCode int) {
	lrw.written = true
	lrw.WrapStatusCode = statusCode
	if lrw.wrapped == nil {
		return
	}
	lrw.wrapped.WriteHeader(statusCode)
}

// Additional functions

// IsWritten returns if lrw has response recorded
func (lrw *WrapResponseWriter) IsWritten() bool {
	return lrw.written
}
