// Copyright (c) 2020 NetApp, Inc. All rights reserved.

package wraphttp

import (
	"bytes"
	"io/ioutil"
	"net/http"

	"github.com/greenqloud/go-lib/uintid"
	"github.com/sirupsen/logrus"
)

// Option is used for configuring the http wrapper
type Option struct {
	FuncPairs []*FuncPair
}

// NewOption returns a new option
func NewOption() *Option {
	return &Option{}
}

// AddFuncPair adds a func pair to opt
// See FuncPair doc for details.
// Each func pair will be excecuted before and after request is handled.
// ResponseWriter and Request will be passed along, after execute func pair functions
// one by one, and changes to them are visible at each stage.
func (o *Option) AddFuncPair(p *FuncPair) *Option {
	o.FuncPairs = append(o.FuncPairs, p)
	return o
}

// WithLogger set the option to include logging with this logger
// Each http request will increment requestID
func (o *Option) WithLog(logger *logrus.Logger, requestID *uintid.UintID) *Option {

	beforeFunc := WrapBeforeFunc(func(w http.ResponseWriter, r *http.Request) (http.ResponseWriter, *http.Request, interface{}) {

		// get the request number
		localReqNum := requestID.Inc()
		r = r.WithContext(uintid.WithRqId(r.Context(), localReqNum))

		// log request
		logger.SetReportCaller(false)
		defer logger.SetReportCaller(true)

		// capture body
		// if read fails bodyBytes will be empty slice and server should handle it.
		bodyBytes, _ := ioutil.ReadAll(r.Body)
		_ = r.Body.Close()

		// construct body again and pass to server backend
		r.Body = ioutil.NopCloser(bytes.NewBuffer(bodyBytes))

		logger.
			WithField("Request Num", localReqNum).
			WithField("Method", r.Method).
			WithField("URL", r.URL).
			WithField("Protocol", r.Proto).
			// WithField("Request Header", r.Header). this will log auth info which is not wanted
			WithField("Request Body", string(bodyBytes)).
			Infof("New Request #%d", localReqNum)

		// wrap to capture response
		lrw := NewLoggingResponseWriter(w)
		return lrw, r, lrw
	})

	afterFunc := WrapAfterFunc(func(w http.ResponseWriter, r *http.Request, x interface{}) (http.ResponseWriter, *http.Request) {
		lrw := x.(*WrapResponseWriter)

		// id will be 0 if not found
		requestID := uintid.ExtractRequestId(r.Context())
		// do not out put file name since it is clearly from this file
		logger.SetReportCaller(false)
		defer logger.SetReportCaller(true)

		entries := logger.
			WithField("Request Num", requestID).
			WithField("Response Code", lrw.WrapStatusCode).
			WithField("Response Header", lrw.Header())

		// we log the response body info if http method is not GET and level is not debug or trace level
		if r.Method != "GET" || logger.GetLevel() >= logrus.DebugLevel {
			entries = entries.WithField("Response Body", string(lrw.WrapBody))
		}
		entries.Infof("Finished Request #%d", requestID)

		return w, r
	})

	return o.AddFuncPair(&FuncPair{
		BeforeFunc: beforeFunc,
		AfterFunc:  afterFunc,
	})
}
