// Copyright (c) 2020 NetApp, Inc. All rights reserved.

package drp

import (
	"fmt"
	"github.com/greenqloud/hyperscaler-network-client/models/azure"
	"github.com/greenqloud/hyperscaler-network-client/models/common"
	stdLog "log"
	"time"

	"github.com/greenqloud/go-lib/log"
	"github.com/Azure/go-autorest/autorest"
)

// clientProperties defines properties of DRP client to be re-used after client instantiation.
// The values of these are typically cached here after reading from environment.
type clientProperties struct {
	contextTimeoutDuration time.Duration
	authEnabled            bool
	*azure.Credentials
	authServerURL     string
	authServerTimeout int
	configClusterList common.ClusterList
}

// getRetrySendDecorators gets autorest retry send decorators for a drpClient instance.
// Inputs:
//	- retryAttempts: Desired number of retries for requests.
//	- backoffDeltaSec: Incremental Back-off interval between retries
//	- backoffMaxSec: Maximum interval between retries
func getRetrySendDecorators(retryAttempts int, backoffDeltaSec int, backoffMaxSec int) []autorest.SendDecorator {
	return []autorest.SendDecorator{
		// autorest accepts only default go logger. So assign io.writer of custom VP's logger to default go logger io.writer
		autorest.WithLogging(stdLog.New(log.GetLogger().Out, "time=", stdLog.LstdFlags)),
		autorest.DoRetryForStatusCodesWithCap(
			retryAttempts,
			time.Duration(backoffDeltaSec)*time.Second,
			time.Duration(backoffMaxSec)*time.Second,
			autorest.StatusCodesForRetry...),
	}
}

// getAuthenticationHeader gets autorest authentication header for an instance of drpClient.
// Inputs:
//	- authenticator: AAn instance of common.Authenticator interface.
// Outputs:
//	- Error, if any.
func getAuthenticationHeader(authenticator common.Authenticator) (autorest.PrepareDecorator, error) {
	if authenticator == nil {
		return nil, fmt.Errorf("nil authenticator input detected while trying to retrieve authentication header")
	}

	token, err := authenticator.GetAuthenticationToken()
	if err != nil {
		return nil, fmt.Errorf("error while retrieving authentication token for DRP request : %v", err)
	}

	// Return dummy autorest.PrepareDecorator in case of mock token.
	if token == common.MockTokenValue {
		return autorest.WithNothing(), nil
	}

	const authorizationHeaderName = "Authorization"
	return autorest.WithHeader(authorizationHeaderName, token), nil
}
