// Copyright (c) 2020-2021 NetApp, Inc. All rights reserved.

package drp

import (
	"context"
	"fmt"
	"net/http"

	v1 "github.com/greenqloud/drp-rest-client/v1/client"
	v2 "github.com/greenqloud/drp-rest-client/v2/client"
	v3 "github.com/greenqloud/drp-rest-client/v3/client"
	"github.com/greenqloud/go-lib/log"
	"github.com/Azure/go-autorest/autorest"

	"github.com/greenqloud/hyperscaler-network-client/models/azure"
	"github.com/greenqloud/hyperscaler-network-client/models/common"
)

// drpClientInterface defines common methods for different DRP client versions.
// Each version of DRP client implements this interface.
type drpClientInterface interface {
	getResponse(ctx context.Context) (response interface{}, err error)
	getClientProperties() *clientProperties
	getAuthenticator() common.Authenticator
	setRequestInspector(requestInspector autorest.PrepareDecorator)
}

// getWithRetry function gets response from a drpClientInterface instance,
// retrying in case authentication token has expired.
// Remarks: This function tries to GET with the existing authentication header attached to the client.
// In case the request yields a 401/unauthorized, a new authentication token is obtained and the GET request is sent again.
// Inputs:
//  ctx: Context for the method
//	client: An instance of drpClientInterface.
//	version: The version of DRP client.
// Outputs:
//	- Response as a generic interface.
//	- Error, if any.
func getWithRetry(ctx context.Context, client drpClientInterface, version string) (interface{}, error) {
	properties := client.getClientProperties()
	if !properties.authEnabled {
		log.Info("Authentication not enabled on client. Attempting to send request without authentication header.")
		return client.getResponse(ctx)
	}

	log.Info("Attempting to send request with the existing authentication token.")
	response, getErr := client.getResponse(ctx)

	// Return early if no error.
	if getErr == nil {
		return response, getErr
	}

	// parse and check status code in case of error
	statusCode, parseErr := tryGetResponseStatusCode(response, version)
	if parseErr != nil {
		log.Errorf("error parsing response from DRP, check if DRP version is supported: %v", parseErr)
		return nil, parseErr
	}

	// return as it is in case of errors other than unauthorized error.
	if statusCode != http.StatusUnauthorized {
		return response, getErr
	}

	// Retry if we get a response with unauthorized status code.
	log.Info("authentication token has expired. Obtaining a new authentication token.")
	authRequestInspector, authErr := getAuthenticationHeader(client.getAuthenticator())

	if authErr != nil {
		log.Errorf("error while obtaining new authentication token for DRP request: %v", authErr)
		return nil, authErr
	}
	client.setRequestInspector(authRequestInspector)

	log.Info("sending request with new authentication token.")
	return client.getResponse(ctx)
}

// tryGetResponseStatusCode tries to extracts the HTTP response code from the given response interface.
// Inputs:
//	- response: Interface representing the response to extract status code from.
//	- version: Version of DRP client corresponding to the response interface.
// Outputs:
//	- An HTTP status code on successful cast of the response interface.
//	- A flag to indicate if response interface can be cast into the given client version.
func tryGetResponseStatusCode(response interface{}, version string) (int, error) {
	// Can not be cast if response is nil.
	if response == nil {
		return 0, common.NewConversionError("Can not extract status code from nil response")
	}

	switch version {
	case azure.V1APIVersion:
		getClusterResult, ok := response.(v1.GetClusterResult)
		if !ok {
			msg := fmt.Sprintf("Error casting to %v DRP response", azure.V1APIVersion)
			return 0, common.NewConversionError(msg)
		}
		if getClusterResult.Response.Response == nil {
			return 0, common.NewConversionError("Cannot extract status code from nil response")
		}
		return getClusterResult.StatusCode, nil
	case azure.V2APIVersion:
		getClusterResult, ok := response.(v2.GetClusterResult)
		if !ok {
			msg := fmt.Sprintf("Error casting to %v DRP response", azure.V2APIVersion)
			return 0, common.NewConversionError(msg)
		}
		if getClusterResult.Response.Response == nil {
			return 0, common.NewConversionError("Cannot extract status code from nil response")
		}
		return getClusterResult.StatusCode, nil
	case azure.V3APIVersion:
		getClusterResult, ok := response.(v3.GetClusterResult)
		if !ok {
			msg := fmt.Sprintf("Error casting to %v DRP response", azure.V3APIVersion)
			return 0, common.NewConversionError(msg)
		}
		if getClusterResult.Response.Response == nil {
			return 0, common.NewConversionError("Cannot extract status code from nil response")
		}
		return getClusterResult.StatusCode, nil
	default:
		return 0, common.NewConversionError("Can not extract status code from unsupported client version")
	}
}
