// Copyright (c) 2020 NetApp, Inc. All rights reserved.

package drp

import (
	"fmt"

	"github.com/greenqloud/go-lib/log"

	"github.com/greenqloud/hyperscaler-network-client/models/common"
)

const (
	// Default value to use for VrfLimit if it is not included in the DRP response. The choice of zero
	// here effectively blocks deployment under this DRP.
	defaultVrfLimit int64 = 0

	// Default value to use for AllocatedRoutes if it si not included in the DRP response. This choice of zero
	// here effectively blocks deployment under this DRP.
	defaultRouteLimit int64 = 0
)

// drpUsageLimit type defines usage limits with types corresponding to actual DRP response.
type drpUsageLimit struct {
	vrfCount        *int32
	vrfLimit        *int32
	routeCount      *int32
	allocatedRoutes *int32
}

// toResponseModel method provides a models.UsageLimit representation of
// current drpUsageLimit instance.
// Output:
//	- models.UsageLimit representation of a drpUsageLimit type.
func (ul *drpUsageLimit) toResponseModel() common.UsageLimitMap {
	var vrfCount, vrfLimit, routeCount, routeLimit int64
	ulMap := make(common.UsageLimitMap)

	if ul.vrfCount != nil {
		vrfCount = int64(*ul.vrfCount)
	} else {
		log.Errorf("Nil value for VRF usage, assuming zero\n")
	}

	// If the limit value is missing we are assuming a default value. See constants above
	// for a description.
	if ul.vrfLimit != nil {
		vrfLimit = int64(*ul.vrfLimit)
	} else {
		log.Errorf("Nil value for VRF limit, assuming default value %d\n", defaultVrfLimit)
		vrfLimit = defaultVrfLimit
	}

	ulMap[common.VRFResourceName] = common.UsageLimit{
		Usage: vrfCount,
		Limit: vrfLimit,
	}

	if ul.routeCount != nil {
		routeCount = int64(*ul.routeCount)
	} else {
		log.Errorf("Nil value for RouteCount, assuming zero\n")
	}

	// If the limit value is missing we are assuming a default value. See constants above
	// for a description.
	if ul.allocatedRoutes != nil {
		routeLimit = int64(*ul.allocatedRoutes)
	} else {
		log.Errorf("Nil value for AllocatedRoutes, assuming default value %d\n", defaultRouteLimit)
		routeLimit = defaultRouteLimit
	}

	ulMap[common.RouteResourceName] = common.UsageLimit{
		Usage: routeCount,
		Limit: routeLimit,
	}

	return ulMap
}

// selectSingleCluster function selects and returns only the relevant cluster standard response from a slice of cluster responses.
// Inputs:
//	- clusterName: Name of the desired cluster.
//	- responseModelClusters: The slice responses from all clusters, in the  models.HyperscalerGetResponse format.
// Outputs:
//	- Response of the desired cluster only
//	- Error, if any.
func selectSingleCluster(clusterName string, responseModelClusters []*common.HyperscalerGetResponse) (*common.HyperscalerGetResponse, error) {
	for _, cluster := range responseModelClusters {
		if cluster.Name == clusterName {
			return cluster, nil
		}
	}

	return nil, fmt.Errorf("no TOR matches the required cluster name %v : %v", clusterName, common.NewClusterNotFoundInNetworkError())
}
