// Copyright (c) 2020 NetApp, Inc. All rights reserved.

package drp

import (
	"fmt"
	"time"

	"github.com/greenqloud/hyperscaler-network-client/models/azure"
	"github.com/greenqloud/hyperscaler-network-client/models/common"

	v1 "github.com/greenqloud/drp-rest-client/v1/client"
	"github.com/greenqloud/go-lib/log"
)

// v1Client type wraps around the DRP client for version - 2018-09-01-preview.
type v1Client struct {
	drp           *v1.ClustersClient
	properties    *clientProperties
	authenticator common.Authenticator
}

// NewV1Client creates a new instance of DRP client V1 with the provided configuration parameters.
// Note: An authorizer is generated and added to the client instance.
// Inputs:
//	- config: An instance of drpConfiguration consisting of all DRP configuration parameters.
//	- region: Region for this client.
// Outputs:
//	- A new instance of DRP V1 client
//	- Error.
func NewV1Client(config *azure.DRPConfiguration, region string, authenticator common.Authenticator) (*v1Client, error) {
	clusterClient := v1.NewClustersClientWithBaseURI(config.ClientConfig.BaseURI, config.ClientConfig.Vnet, region)
	v1ClientInstance := &v1Client{drp: &clusterClient, properties: &clientProperties{}}

	v1ClientInstance.properties.contextTimeoutDuration = time.Duration(config.ClientConfig.RequestTimeoutDuration) * time.Second
	v1ClientInstance.properties.configClusterList = config.ConfigClusterList

	v1ClientInstance.drp.SendDecorators = getRetrySendDecorators(
		config.ClientConfig.RetryAttempts,
		config.ClientConfig.RetryBackoffDeltaSec,
		config.ClientConfig.RetryBackoffMaxSec)

	if !config.ClientConfig.AuthenticationEnabled {
		log.Info("Authentication disabled for client.")
		v1ClientInstance.properties.authEnabled = false
		return v1ClientInstance, nil
	}

	if authenticator == nil {
		msg := "authentication enabled on client but nil authenticator input detected"
		log.Error(msg)
		return nil, fmt.Errorf(msg)
	}

	authRequestInspector, err := getAuthenticationHeader(authenticator)
	if err != nil {
		return nil, fmt.Errorf("error in retrieving authentication header: %v", err)
	}
	v1ClientInstance.authenticator = authenticator

	log.Info("Authentication enabled for client and authentication properties attached.")
	v1ClientInstance.properties.authEnabled = true
	v1ClientInstance.properties.Credentials = config.Credentials
	v1ClientInstance.properties.authServerURL = config.AuthConfig.AuthServerURL
	v1ClientInstance.properties.authServerTimeout = config.AuthConfig.TimeoutDuration
	v1ClientInstance.drp.RequestInspector = authRequestInspector

	return v1ClientInstance, nil
}
