// Copyright (c) 2020 NetApp, Inc. All rights reserved.

package drp

import (
	"context"
	"fmt"
	"net/http"

	v1 "github.com/greenqloud/drp-rest-client/v1/client"
	"github.com/greenqloud/go-lib/log"
	"github.com/greenqloud/go-lib/render"
	"github.com/Azure/go-autorest/autorest"

	"github.com/greenqloud/hyperscaler-network-client/models/azure"
	"github.com/greenqloud/hyperscaler-network-client/models/common"
)

// getResponse gets DRP V1 client response.
// Note: Implements the internal drpClientInterface interface on DRP V1 client.
// Receiver:
//	c: DRP V1 client pointer.
// Outputs:
//	- V1 client response in generic interface format.
//	- Error, if any.
func (c *v1Client) getResponse(ctx context.Context) (response interface{}, err error) {
	return c.drp.List(ctx)
}

// getResponse gets DRP V1 client properties.
// Note: Implements the internal drpClientInterface interface on DRP V1 client.
// Receiver:
//	c: DRP V1 client pointer.
// Outputs:
//	- V1 client properties.
func (c *v1Client) getClientProperties() *clientProperties {
	return c.properties
}

// getAuthenticator get common.Authenticator associated with the DRP V1 client instance
// Note: Implements the internal drpClientInterface interface on DRP V1 client.
// Receiver:
//	c: DRP V1 client pointer.
// Outputs:
//	- common.Authenticator
func (c *v1Client) getAuthenticator() common.Authenticator {
	return c.authenticator
}

// getResponse sets request inspector on DRP V1 client.
// Note: Implements the internal drpClientInterface interface on DRP V1 client.
// Receiver:
//	c: DRP V1 client pointer.
func (c *v1Client) setRequestInspector(requestInspector autorest.PrepareDecorator) {
	c.drp.RequestInspector = requestInspector
}

// PreProcessClient performs region pre-processing on DRP V1 client
// Note: Implements models.HyperscalerClusterGetter interface on v1Client.
// Receiver:
//	c: DRP V1 client pointer.
func (c *v1Client) PreProcessClient(regionName string) {
	if c.drp.Region != "" {
		log.Infof("using the region set in environment: %v", c.drp.Region)
	} else {
		c.drp.Region = regionName
		log.Infof("DRP client region set to %v", regionName)
	}
}

// GetClusters lists all clusters for the client.
// Note: Implements models.HyperscalerClusterGetter interface on v1Client.
// Receiver:
//	c: DRP V1 client pointer.
// Outputs:
//	- Response from all clusters in models.HyperscalerGetResponse format.
//	- Error.
func (c *v1Client) GetClusters() ([]*common.HyperscalerGetResponse, error) {
	ctx, cancel := context.WithTimeout(context.Background(), c.properties.contextTimeoutDuration)
	defer cancel()
	return c.getClusters(ctx)
}

// GetClustersWithContext lists all clusters for the client with the provided context.
// Note: Implements models.HyperscalerClusterGetter interface on v1Client.
// Receiver:
//	c: DRP V1 client pointer.
// Input:
//  ctx: context for the method
// Outputs:
//	- Response from all clusters in models.HyperscalerGetResponse format.
//	- Error.
func (c *v1Client) GetClustersWithContext(ctx context.Context) ([]*common.HyperscalerGetResponse, error) {
	return c.getClusters(ctx)
}

// GetCluster gets the required cluster for the client.
// Note: Implements models.HyperscalerClusterGetter interface on v1Client.
// Receiver:
//	c: DRP V1 client pointer.
// Outputs:
//	- Required cluster response in models.HyperscalerGetResponse format.
//	- Error, if any.
func (c *v1Client) GetCluster(clusterName string) (*common.HyperscalerGetResponse, error) {
	ctx, cancel := context.WithTimeout(context.Background(), c.properties.contextTimeoutDuration)
	defer cancel()
	return c.GetClusterWithContext(ctx, clusterName)
}

// GetClusterWithContext gets the required cluster for the client using the provided context.
// Note: Implements models.HyperscalerClusterGetter interface on v1Client.
// Receiver:
//	c: DRP V1 client pointer.
// Input:
//  ctx: context for the method
// Outputs:
//	- Required cluster response in models.HyperscalerGetResponse format.
//	- Error, if any.
func (c *v1Client) GetClusterWithContext(ctx context.Context, clusterName string) (*common.HyperscalerGetResponse, error) {
	return c.getCluster(ctx, clusterName)
}

// getClusters helper method which lists all clusters for the client.
// Receiver:
//	c: DRP V1 Client pointer.
// Input:
//  ctx: context for the method
// Outputs:
//	- Response from all clusters in models.HyperscalerGetResponse format.
//	- Error.
func (c *v1Client) getClusters(ctx context.Context) ([]*common.HyperscalerGetResponse, error) {
	resp, err := getWithRetry(ctx, c, azure.V1APIVersion)
	if err != nil {
		log.Error(err)
		return nil, fmt.Errorf("error while retrieving information from DRP client: %v", err)
	}

	result := resp.(v1.GetClusterResult)

	if result.Response.StatusCode != http.StatusOK {
		return nil, fmt.Errorf("error in response from drp client: %v, %v", common.NewBadRequestError("Error in response from DRP client"), result.Response)
	}

	var responseClusters []*common.HyperscalerGetResponse
	if result.Clusters == nil || len(*result.Clusters) == 0 {
		if len(c.getClientProperties().configClusterList.Clusters) == 0 {
			return nil, fmt.Errorf("no hyperscaler cluster found in network: %v", common.NewNoHyperscalerClustersFoundInNetwork())
		}
	} else {
		responseClusters = convertV1ClustersToResponseModel(*result.Clusters)
	}
	if len(c.properties.configClusterList.Clusters) == 0 {
		if len(responseClusters) == 0 {
			return nil, fmt.Errorf("unable to convert any DRP cluster to standard response format; %v", common.NewDRPResponseConversionError())
		}
		return responseClusters, nil
	}

	log.Debug("Loading config cluster")
	// Construct V1 clusterResponse from
	var v1Clusters []v1.Cluster
	for _, cluster := range c.properties.configClusterList.Clusters {
		var v1Cluster = v1.Cluster{
			ClusterName:     &cluster.ClusterName,
			VrfCount:        &cluster.VrfCount,
			VrfLimit:        &cluster.VrfLimit,
			RouteCount:      &cluster.RouteCount,
			RouteLimit:      &cluster.RouteLimit,
			AllocatedRoutes: &cluster.AllocatedRoutes,
			Tier2alias:      &cluster.Tier2Alias,
		}
		v1Clusters = append(v1Clusters, v1Cluster)
	}
	configClusters := convertV1ClustersToResponseModel(v1Clusters)
	return common.MergeUniqueHyperscalerResponse(responseClusters, configClusters), nil
}

// getCluster helper method to get the required cluster for the client using the provided context.
// Receiver:
//	c: DRP V1 client pointer.
// Input:
//  ctx: context for the method
// Outputs:
//	- Required cluster response in models.HyperscalerGetResponse format.
//	- Error, if any.
func (c *v1Client) getCluster(ctx context.Context, clusterName string) (*common.HyperscalerGetResponse, error) {
	responseModelClusters, err := c.getClusters(ctx)
	if err != nil {
		return nil, err
	}

	// result.Clusters != nil is checked in getAllTORs
	return selectSingleCluster(clusterName, responseModelClusters)
}

// convertV1ClustersToResponseModel converts all clusters fro DRP V1 client response to models.HyperscalerGetResponse format.
// Inputs:
//	- clusters: Slice of clusters received from DRP V1 client as response.
func convertV1ClustersToResponseModel(clusters []v1.Cluster) []*common.HyperscalerGetResponse {
	var convertedClusters []*common.HyperscalerGetResponse

	for _, cluster := range clusters {
		log.Debug(render.Content(cluster))

		var clusterName string
		tags := make(map[common.Tag]*string)

		if cluster.ClusterName != nil {
			clusterName = *cluster.ClusterName
		} else {
			log.Error("Cluster with nil cluster name detected")
		}

		drpUl := &drpUsageLimit{
			vrfCount:        cluster.VrfCount,
			vrfLimit:        cluster.VrfLimit,
			routeCount:      cluster.RouteCount,
			allocatedRoutes: cluster.AllocatedRoutes,
		}

		usageLimit := drpUl.toResponseModel()

		// Initialize network info
		network := &common.Network{}

		if cluster.Tier2alias != nil {
			tags[common.NetworkProximity] = cluster.Tier2alias
			network.Proximity = cluster.Tier2alias
		} else {
			log.Error("Nil pointer for tier2Alias detected")
		}

		// set default network info
		network.Connectivity = common.NetworkConnectivityDefault
		network.Type = common.NetworkTypeBasic
		network.SubType = common.NetworkSubtypeDefault

		newCluster := &common.HyperscalerGetResponse{
			Name:          clusterName,
			UsageLimitMap: usageLimit,
			TagValues:     tags,
			Network:       network,
		}
		convertedClusters = append(convertedClusters, newCluster)
	}

	return convertedClusters
}
