// Copyright (c) 2020 NetApp, Inc. All rights reserved.

package drp

import (
	"fmt"
	"time"

	"github.com/greenqloud/hyperscaler-network-client/models/azure"
	"github.com/greenqloud/hyperscaler-network-client/models/common"

	v2 "github.com/greenqloud/drp-rest-client/v2/client"
	"github.com/greenqloud/go-lib/log"
)

// v2Client type wraps around the DRP client for version - 2020-10-01-preview.
type v2Client struct {
	drp           *v2.ClustersClient
	properties    *clientProperties
	authenticator common.Authenticator
}

// NewV2Client creates a new instance of DRP client V2 with the provided configuration parameters.
// Note: An authorizer is generated and added to the client instance.
// Inputs:
//	- config: An instance of drpConfiguration consisting of all DRP configuration parameters.
//	- region: Region for this client.
// Outputs:
//	- A new instance of DRP V2 client
//	- Error.
func NewV2Client(config *azure.DRPConfiguration, region string, authenticator common.Authenticator) (*v2Client, error) {
	clusterClient := v2.NewClustersClientWithBaseURI(config.ClientConfig.BaseURI, config.ClientConfig.Vnet, region)
	v2ClientInstance := &v2Client{drp: &clusterClient, properties: &clientProperties{}}

	v2ClientInstance.properties.contextTimeoutDuration = time.Duration(config.ClientConfig.RequestTimeoutDuration) * time.Second
	v2ClientInstance.properties.configClusterList = config.ConfigClusterList

	v2ClientInstance.drp.SendDecorators = getRetrySendDecorators(
		config.ClientConfig.RetryAttempts,
		config.ClientConfig.RetryBackoffDeltaSec,
		config.ClientConfig.RetryBackoffMaxSec)

	if !config.ClientConfig.AuthenticationEnabled {
		log.Info("Authentication disabled for client.")
		v2ClientInstance.properties.authEnabled = false
		return v2ClientInstance, nil
	}

	if authenticator == nil {
		msg := "authentication enabled on client but nil authenticator input detected"
		log.Error(msg)
		return nil, fmt.Errorf(msg)
	}

	authRequestInspector, err := getAuthenticationHeader(authenticator)
	if err != nil {
		return nil, fmt.Errorf("error in retrieving authentication header: %v", err)
	}
	v2ClientInstance.authenticator = authenticator

	log.Info("Authentication enabled for client and authentication properties attached.")
	v2ClientInstance.properties.authEnabled = true
	v2ClientInstance.properties.Credentials = config.Credentials
	v2ClientInstance.properties.authServerURL = config.AuthConfig.AuthServerURL
	v2ClientInstance.properties.authServerTimeout = config.AuthConfig.TimeoutDuration
	v2ClientInstance.drp.RequestInspector = authRequestInspector

	return v2ClientInstance, nil
}
