// Copyright (c) 2021 NetApp, Inc. All rights reserved.

package drp

import (
	"fmt"
	"time"

	"github.com/greenqloud/hyperscaler-network-client/models/azure"
	"github.com/greenqloud/hyperscaler-network-client/models/common"

	v3 "github.com/greenqloud/drp-rest-client/v3/client"
	"github.com/greenqloud/go-lib/log"
)

// v3Client type wraps around the DRP client for version - 2021-06-01-preview.
type v3Client struct {
	drp           *v3.ClustersClient
	properties    *clientProperties
	authenticator common.Authenticator
}

// NewV3Client creates a new instance of DRP client V3 with the provided configuration parameters.
// Note: An authorizer is generated and added to the client instance.
// Inputs:
//	- config: An instance of drpConfiguration consisting of all DRP configuration parameters.
//	- region: Region for this client.
// Outputs:
//	- A new instance of DRP V3 client
//	- Error.
func NewV3Client(config *azure.DRPConfiguration, region string, authenticator common.Authenticator) (*v3Client, error) {
	clusterClient := v3.NewClustersClientWithBaseURI(config.ClientConfig.BaseURI, config.ClientConfig.Vnet, region)
	v3ClientInstance := &v3Client{drp: &clusterClient, properties: &clientProperties{}}

	v3ClientInstance.properties.contextTimeoutDuration = time.Duration(config.ClientConfig.RequestTimeoutDuration) * time.Second
	v3ClientInstance.properties.configClusterList = config.ConfigClusterList

	v3ClientInstance.drp.SendDecorators = getRetrySendDecorators(
		config.ClientConfig.RetryAttempts,
		config.ClientConfig.RetryBackoffDeltaSec,
		config.ClientConfig.RetryBackoffMaxSec)

	if !config.ClientConfig.AuthenticationEnabled {
		log.Info("Authentication disabled for client.")
		v3ClientInstance.properties.authEnabled = false
		return v3ClientInstance, nil
	}

	if authenticator == nil {
		msg := "authentication enabled on client but nil authenticator input detected"
		log.Error(msg)
		return nil, fmt.Errorf(msg)
	}

	authRequestInspector, err := getAuthenticationHeader(authenticator)
	if err != nil {
		return nil, fmt.Errorf("error in retrieving authentication header: %v", err)
	}
	v3ClientInstance.authenticator = authenticator

	log.Info("Authentication enabled for client and authentication properties attached.")
	v3ClientInstance.properties.authEnabled = true
	v3ClientInstance.properties.Credentials = config.Credentials
	v3ClientInstance.properties.authServerURL = config.AuthConfig.AuthServerURL
	v3ClientInstance.properties.authServerTimeout = config.AuthConfig.TimeoutDuration

	v3ClientInstance.drp.RequestInspector = authRequestInspector

	return v3ClientInstance, nil
}
