// Copyright (c) 2020 NetApp, Inc. All rights reserved.

// Package dotnet provides authentication by the ASP .NET core project - azure authentication client
// see https://git.greenqloud.com/team-svl/azure-auth-client
package dotnet

import (
	"bytes"
	"fmt"
	"github.com/greenqloud/hyperscaler-network-client/models/azure"
	"io/ioutil"
	"net/http"
	"time"

	"github.com/greenqloud/go-lib/log"
)

const (
	contentTypeKey        = "Content-Type"     // content type header key
	jsonContentType       = "application/json" // value representing json content type
	noExceptionInResponse = "null"             // Unmarshalled string representation of no exception in response for azure-auth-server
)

// authClient composed of AzureCredentials and HTTP Client.
// Performs necessary steps to communicate with the .NET API
type authClient struct {
	*azure.Credentials
	*http.Client
	Endpoint string
}

// NewAuthClient creates a new instance of authClient type.
// Input:
//	- credentials: Pointer to AzureCredentials type.
//	- url: Endpoint of the dotnet authentication API server.
//	- timeout: Duration after which requests to the client time out.
func NewAuthClient(credentials *azure.Credentials, url string, timeout time.Duration) *authClient {
	httpCli := &http.Client{}
	httpCli.Timeout = timeout
	authClient := &authClient{Credentials: credentials, Client: httpCli, Endpoint: url}

	return authClient
}

// GetAuthenticationToken gets the authentication token from .NET authentication API server.
// Remarks: This method POSTs the AzureCredentials data to the authentication server. The server responds back with the bearer token and exceptions if any.
// This method processes the response from server and returns the authentication token or an error.
// Receiver:
//	- a : authClient type pointer.
// Output:
//	- authentication token string.
//	- Error, if any.
func (a *authClient) GetAuthenticationToken() (string, error) {
	req, err := a.createPostRequest()
	if err != nil {
		return "", err
	}

	resp, err := a.Do(req)
	if err != nil {
		return "", err
	}
	defer resp.Body.Close()

	if resp.StatusCode >= http.StatusBadRequest {
		return "", fmt.Errorf("error in response from auzre authentication client: %v", resp.StatusCode)
	}

	respBytes, err := ioutil.ReadAll(resp.Body)
	if err != nil {
		return "", err
	}

	authResponse, err := unmarshalIntoAuthenticatorResponse(respBytes)
	if err != nil {
		return "", err
	}

	exceptionString := string(authResponse.Exception)
	if exceptionString != noExceptionInResponse {
		return "", fmt.Errorf("azure authentication client error : %v", exceptionString)
	}

	log.Info("authentication token obtained successfully")
	return authResponse.BearerToken, nil
}

// createPostRequest uses the embedded AzureCredentials and Endpoint in the authClient object to create an HTTP post request.
// Receiver:
//	- a : authClient type pointer.
//	Output:
//	- An HTTP request.
//	- Error, if any.
func (a *authClient) createPostRequest() (*http.Request, error) {
	jsonString, err := a.ToJSONString()
	if err != nil {
		return nil, err
	}

	jsonBytes := []byte(jsonString)
	req, err := http.NewRequest(http.MethodPost, a.Endpoint, bytes.NewBuffer(jsonBytes))
	if err != nil {
		return nil, err
	}

	req.Header.Set(contentTypeKey, jsonContentType)

	log.Info("request to post credentials created successfully")
	return req, nil
}
