package keyvault

import (
	"context"
	"encoding/base64"
	"fmt"
	"os"

	kvauth "github.com/Azure/azure-sdk-for-go/services/keyvault/auth"
	akv "github.com/Azure/azure-sdk-for-go/services/keyvault/v7.0/keyvault"
	"github.com/Azure/go-autorest/autorest"
	"github.com/Azure/go-autorest/autorest/adal"

	"github.com/greenqloud/go-lib/log"
)

// secretProvider interface defines contract for obtaining secret from a URL
type secretProvider interface {
	GetSecret(ctx context.Context, baseURL string, secretName string, secretVersion string) (akv.SecretBundle, error)
}

// Client type encapsulates a keyvault.BaseClient
type Client struct {
	secretProvider secretProvider
}

// NewClientFromSP creates a new instance of Client from the given Credentials using Service Principal.
func NewClientFromSP(kvCredentials *Credentials) (*Client, error) {
	if kvCredentials == nil {
		msg := "keyvault client can not be created with nil credentials"
		log.Error(msg)
		return nil, fmt.Errorf(msg)
	}

	log.Info("creating OAuth configuration for keyvault")
	kvConfig, err := adal.NewOAuthConfig(kvCredentials.ServicePrincipalInfo.AuthorityEndPoint, kvCredentials.ServicePrincipalInfo.TenantID)
	if err != nil {
		log.Errorf("error while creating OAuth config for key vault. Check keyvault authority and tenant ID: %v", err)
		return nil, err
	}

	log.Info("obtaining service principal token for keyvault")
	token, err := adal.NewServicePrincipalToken(*kvConfig, kvCredentials.ServicePrincipalInfo.ClientID, kvCredentials.secret, kvCredentials.ServicePrincipalInfo.ResourceEndPoint)
	if err != nil {
		log.Errorf("error while retrieving service principal token for keyvault: %v", err)
		return nil, err
	}

	log.Info("setting authorizer on new keyvault client instance")
	kv := akv.New()
	kv.Authorizer = autorest.NewBearerAuthorizer(token)

	log.Info("keyvault client instance using SP initialized.")
	return &Client{secretProvider: kv}, nil
}

// NewClient creates a new instance of Client from the given Credentials.
func NewClientFromMSI(msiManagedIdentityID string) (*Client, error) {
	// set AZURE_CLIENT_ID env required by kvauth to create authorizer
	os.Setenv("AZURE_CLIENT_ID", msiManagedIdentityID)

	authorizer, err := kvauth.NewAuthorizerFromEnvironment()
	if err != nil {
		log.Error("Unable to create vault authorizer from environment: ", err.Error())
		return nil, err
	}

	log.Info("setting authorizer on new keyvault client instance")
	kv := akv.New()
	kv.Authorizer = authorizer

	log.Info("keyvault client instance using MSI initialized.")
	return &Client{secretProvider: kv}, nil
}

// GetCertificate retrieves the required certificate and private key from a keyvault - given its base URL.
func (c *Client) GetCertificate(ctx context.Context, kvURL string, certificateName string) (*CertKeyBundle, error) {
	log.Info("getting secret from keyvault")
	secret, err := c.secretProvider.GetSecret(ctx, kvURL, certificateName, "")
	if err != nil {
		log.Errorf("error while getting secret from keyvault. Check keyvault credentials and info: %v", err)
		return nil, err
	}

	return extractCertKeyBundleFromSecretString(*secret.Value)
}

// extractCertKeyBundleFromSecretString extracts the x509.Certificate and rsa.PrivateKey bundle
// from a base 64 encoded secret string.
func extractCertKeyBundleFromSecretString(secretValue string) (*CertKeyBundle, error) {
	log.Info("performing base64 Decoding of secret")
	pfxBytes, err := base64.StdEncoding.DecodeString(secretValue)
	if err != nil {
		log.Errorf("error in decoding secret - base 64 decode of secret failed with error: %v", err)
		return nil, err
	}

	log.Info("converting raw pfx data in secret to certificate")
	certificate, rsaPrivateKey, err := adal.DecodePfxCertificateData(pfxBytes, "")
	if err != nil {
		log.Errorf("error while extracting certificate from raw keyvault secret data. Failed with error: %v", err)
		return nil, err
	}

	log.Info("certificate and private key bundle obtained from keyvault.")
	return &CertKeyBundle{
		Cert: certificate,
		Key:  rsaPrivateKey,
	}, nil

}
