package keyvault

import (
	"fmt"

	"github.com/greenqloud/hyperscaler-network-client/models/azure"
	"github.com/greenqloud/hyperscaler-network-client/models/common"

	goazure "github.com/Azure/go-autorest/autorest/azure"
)

// ResourceType defines different types of azure resources
type ResourceType string

const (
	// Types of resources
	KeyVault                     ResourceType = "KeyVault"
	DirectAttachResourceProvider ResourceType = "DirectAttachResourceProvider"
)

// Credentials defines credentials required to access an azure keyvault.
type Credentials struct {
	ServicePrincipalInfo ServicePrincipalInfo
	secret               string
}

// ServicePrincipalInfo type defines the service principal identity
// required to access different azure resources.
type ServicePrincipalInfo struct {
	TenantID          string
	ClientID          string
	AuthorityEndPoint string
	ResourceEndPoint  string
}

// ExtractVaultCredentials extracts keyvault Credentials from the
// given super-set of azure.Credentials.
func ExtractVaultCredentials(credentials *azure.Credentials) (*Credentials, error) {
	sp, err := GetServicePrincipalInfo(credentials, KeyVault)
	if err != nil {
		return nil, err
	}
	return &Credentials{
		ServicePrincipalInfo: *sp,
		secret:               credentials.ManagedIdentitySecret,
	}, nil
}

// GetServicePrincipalInfo extracts the ServicePrincipalInfo for the required ResourceType
// from given instance of Credentials.
func GetServicePrincipalInfo(credentials *azure.Credentials, resourceType ResourceType) (*ServicePrincipalInfo, error) {
	authorityEndpoint, drpResourceEndpoint, kvResourceEndpoint := getAzureEndpoints(credentials.AzureEnvironment)

	switch resourceType {
	case KeyVault:
		msiConfig, err := azure.NewMSIConfigFromEnvironment()
		if err != nil {
			return nil, err
		}
		// Use MsiManagedIdentityID as client ID if msiAuth is enabled, else use SpManagedIdentityID
		clientID := credentials.SpManagedIdentityID
		if msiConfig.MsiAuthEnabled {
			clientID = credentials.MsiManagedIdentityID
		}
		return &ServicePrincipalInfo{
			TenantID:          credentials.TenantID,
			ClientID:          clientID,
			AuthorityEndPoint: authorityEndpoint,
			ResourceEndPoint:  kvResourceEndpoint,
		}, nil
	case DirectAttachResourceProvider:
		return &ServicePrincipalInfo{
			TenantID:          credentials.TenantID,
			ClientID:          credentials.ApplicationID,
			AuthorityEndPoint: authorityEndpoint,
			ResourceEndPoint:  drpResourceEndpoint,
		}, nil
	default:
		msg := fmt.Sprintf("case %v not implemented for retrieving service principal information from environment.", resourceType)
		return nil, common.NewNotImplementedError(msg)
	}
}

// getAzureEndpoints returns the appropriate authority and resource addresses for DRP and Keyvault based on the provided cloud name.
// Input:
//	cloudName: Name of the azure cloud environment.
// Outputs:
// authorityAddress: Authority address for DRP and KV authentication.
// drpResourceAddress: Resource address to use for DRP authentication.
// kvResourceAddress: Resource address to use for KV authentication.
func getAzureEndpoints(environmentName string) (authorityAddress string, drpResourceAddress string, kvResourceAddress string) {
	switch environmentName {
	case goazure.USGovernmentCloud.Name:
		authorityAddress = goazure.USGovernmentCloud.ActiveDirectoryEndpoint
		drpResourceAddress = goazure.USGovernmentCloud.ResourceManagerEndpoint
		kvResourceAddress = goazure.USGovernmentCloud.ResourceIdentifiers.KeyVault
	case goazure.ChinaCloud.Name:
		authorityAddress = goazure.ChinaCloud.ActiveDirectoryEndpoint
		drpResourceAddress = goazure.ChinaCloud.ResourceManagerEndpoint
		kvResourceAddress = goazure.ChinaCloud.ResourceIdentifiers.KeyVault
	case goazure.GermanCloud.Name:
		authorityAddress = goazure.GermanCloud.ActiveDirectoryEndpoint
		drpResourceAddress = goazure.GermanCloud.ResourceManagerEndpoint
		kvResourceAddress = goazure.GermanCloud.ResourceIdentifiers.KeyVault
	default:
		authorityAddress = goazure.PublicCloud.ActiveDirectoryEndpoint
		drpResourceAddress = goazure.PublicCloud.ResourceManagerEndpoint
		kvResourceAddress = goazure.PublicCloud.ResourceIdentifiers.KeyVault
	}
	return
}
