package keyvault

import (
	"context"
	"crypto/rsa"
	"crypto/x509"
	"fmt"
	"github.com/greenqloud/go-lib/log"
	"github.com/greenqloud/hyperscaler-network-client/models/azure"
	"github.com/Azure/go-autorest/autorest/adal"
	"time"
)

const (
	bearerTokenPrefix = "Bearer"
)

// CertKeyBundle represents a bundle of x509.Certificate and rsa.PrivateKey for that certificate.
type CertKeyBundle struct {
	Cert *x509.Certificate
	Key  *rsa.PrivateKey
}

type certificateProvider interface {
	GetCertificate(ctx context.Context, kvURL string, certificateName string) (*CertKeyBundle, error)
}

// tokenProvider type to retrieve authentication token from keyvault.
type tokenProvider struct {
	certProvider certificateProvider
	credentials  *azure.Credentials
	TimeOut      time.Duration
}

// createKVClient creates a keyvault client using MSI or SP based on msi auth enabled flag
func createKVClient(msiConfig *azure.MsiConfig, credentials *azure.Credentials) (*Client, error) {
	if msiConfig.MsiAuthEnabled {
		return NewClientFromMSI(credentials.MsiManagedIdentityID)
	}

	cred, err := ExtractVaultCredentials(credentials)
	if err != nil {
		return nil, err
	}
	return NewClientFromSP(cred)
}

// NewTokenProvider creates a new instance of keyvault tokenProvider with the given azure.Credentials and
// auth timeout duration.
func NewTokenProvider(credentials *azure.Credentials, timeout time.Duration) (*tokenProvider, error) {
	tp := &tokenProvider{
		credentials: credentials,
		TimeOut:     timeout,
	}
	msiConfig, err := azure.NewMSIConfigFromEnvironment()
	if err != nil {
		return nil, err
	}
	cli, err := createKVClient(msiConfig, credentials)
	if err != nil {
		return nil, err
	}
	tp.certProvider = cli
	log.Info("keyvault token provider initialized.")
	return tp, nil
}

// GetAuthenticationToken gets the authentication token from the keyvault.
// Note: Implement common.Authenticator interface on tokenProvider
func (t *tokenProvider) GetAuthenticationToken() (string, error) {
	log.Info("extracting DRP service principal information from credentials")
	sp, err := GetServicePrincipalInfo(t.credentials, DirectAttachResourceProvider)
	if err != nil {
		return "", fmt.Errorf("error in extracting DRP service principal information form given azure credentials: %v", err)
	}

	log.Infof("creating OAuth configuration for DRP with authority: %v", sp.AuthorityEndPoint)
	oauth, err := adal.NewOAuthConfig(sp.AuthorityEndPoint, sp.TenantID)
	if err != nil {
		return "", fmt.Errorf("error while creating OAuth config for DRP. Check DRP authority and tenant ID: %v", err)
	}

	ctx, cancel := context.WithTimeout(context.Background(), t.TimeOut)
	defer cancel()

	log.Info("getting certificate from keyvault")
	cert, err := t.certProvider.GetCertificate(ctx, t.credentials.KeyVaultBaseURL, t.credentials.KeyVaultSecretName)
	if err != nil {
		return "", fmt.Errorf("error while downloading certificate from keyvault: %v", err)
	}

	log.Infof(
		"obtaining service principal token for DRP authentication with certProvider ID = %v and base resource %v",
		sp.ClientID,
		sp.ResourceEndPoint)

	spToken, err := adal.NewServicePrincipalTokenFromCertificate(*oauth, sp.ClientID, cert.Cert, cert.Key, sp.ResourceEndPoint)
	if err != nil {
		return "", fmt.Errorf("error in creating service principal token for DRP authentication. Failed with error: %v", err)
	}

	log.Info("ensuring obtained token is fresh")
	err = spToken.EnsureFreshWithContext(ctx)
	if err != nil {
		return "", fmt.Errorf("error while refershing service principal token for DRP authentication: %v", err)
	}

	log.Info("authentication token obtained successfully")
	return fmt.Sprintf("%s %s", bearerTokenPrefix, spToken.OAuthToken()), nil

}
