// Copyright (c) 2020-2021 NetApp, Inc. All rights reserved.

package factories

import (
	"fmt"

	"github.com/greenqloud/go-lib/log"

	"github.com/greenqloud/hyperscaler-network-client/adapters/drp"
	"github.com/greenqloud/hyperscaler-network-client/models/azure"
	"github.com/greenqloud/hyperscaler-network-client/models/common"
)

// CreateNewDRPClient provides the appropriate DRP client  instance of common.HyperscalerClusterGetter interface.
// Note: Use this method when DRP configurations are already set in environment.
// Outputs:
//	- A DRP client implementation of common.HyperscalerClusterGetter interface.
//	- Error, if any.
func CreateNewDRPClient(configClusterList common.ClusterList) (common.HyperscalerClusterGetter, error) {
	config, err := azure.NewDRPConfiguration()
	if err != nil {
		return nil, fmt.Errorf("error in retrieving required DRP environment variables: %v", err)
	}
	config.ConfigClusterList = configClusterList
	return CreateNewDRPClientFromConfig(config)
}

// CreateNewDRPClientFromConfig provides the appropriate DRP client  instance of common.HyperscalerClusterGetter interface
// based on input azure.DRPConfiguration object.
// Note: Use this method when config not set in environment.
// Outputs:
//	- A DRP client implementation of common.HyperscalerClusterGetter interface.
//	- Error, if any.
func CreateNewDRPClientFromConfig(config *azure.DRPConfiguration) (common.HyperscalerClusterGetter, error) {
	log.Infof("Creating Authenticator for DRP Client")
	authenticator, err := CreateNewDRPAuthenticatorFromConfigOnly(config)
	if err != nil {
		return nil, fmt.Errorf("error in creating authenticator for DRP client: %v", err)
	}

	apiVersion := config.ClientConfig.APIVersion
	log.Infof("Creating DRP client for version: %v", apiVersion)

	switch apiVersion {
	case azure.V1APIVersion:
		return drp.NewV1Client(config, config.ClientConfig.RegionOverride, authenticator)
	case azure.V2APIVersion:
		return drp.NewV2Client(config, config.ClientConfig.RegionOverride, authenticator)
	case azure.V3APIVersion:
		return drp.NewV3Client(config, config.ClientConfig.RegionOverride, authenticator)
	default:
		msg := fmt.Sprintf("DRP client API version %v not supported. Supported versions: %v, %v, %v",
			apiVersion, azure.V1APIVersion, azure.V2APIVersion, azure.V3APIVersion)
		return nil, common.NewNotImplementedError(msg)
	}
}
