// Copyright (c) 2020 NetApp, Inc. All rights reserved.

package factories

import (
	"fmt"

	"github.com/greenqloud/go-lib/log"
	"github.com/greenqloud/go-lib/render"

	"github.com/greenqloud/hyperscaler-network-client/adapters/cvn"
	"github.com/greenqloud/hyperscaler-network-client/models/common"
)

// CreateNewClient function provides an instance of models.HyperscalerClusterGetter based on the input hyperscaler type.
// Inputs:
//	- hyperscalerType: Hyperscaler type. Eg: Azure, GCP etc.
// Outputs:
//	- The appropriate instance of models.HyperscalerClusterGetter interface for the hyperscaler environment.
//	- Error, if any.
func CreateNewClient(hyperscalerType common.HyperscalerType) (common.HyperscalerClusterGetter, error) {
	hyperscalerConfig, err := common.NewHyperscalerConfigFromEnv()
	if err != nil {
		return nil, fmt.Errorf("error in getting hyperscaler config %v", err)
	}
	var configClusterList common.ClusterList
	if hyperscalerConfig.LoadFromPatchFile {
		configClusterList, err = common.LoadClusterListYaml(hyperscalerConfig.PatchFilename)
		if err != nil {
			log.Warnf("Unable to load clusters from config yaml %s, error %v", hyperscalerConfig.PatchFilename, err)
		} else {
			log.Infof("Loading clusters from %s file, with contents", hyperscalerConfig.PatchFilename)
			log.Info(render.Content(configClusterList))
		}
	}
	switch hyperscalerType {
	case common.AzureCluster:
		return CreateNewDRPClient(configClusterList)
	case common.GCPCluster:
		return cvn.NewV1MockClient()
	default:
		msg := fmt.Sprintf("Hyperscaler cluster type  %v not supported", hyperscalerType)
		return nil, common.NewNotImplementedError(msg)
	}
}
