// Copyright (c) 2020 NetApp, Inc. All rights reserved.

package azure

import (
	"encoding/json"
	"github.com/caarlos0/env"
)

// AuthenticationConfig defines the configuration parameters for authenticating requests to DRP.
type AuthenticationConfig struct {
	TimeoutDuration int    `env:"DRP_AUTH_SERVER_TIMEOUT_SEC,required"`
	AuthServerURL   string `env:"DRP_AUTH_SERVER_URL"`
}

// Credentials  represents the set of azure credentials required by the .NET API.
// These credential values can be read from environment.
// ApplicationID: ID used to authenticate to DRP server
// SpManagedIdentityID: ID used to obtain service principal token for keyvault when using SP authentication
// MsiManagedIdentityID: ID used to obtain service principal token for keyvault when using MSI authentication
// ManagedIdentitySecret: secret used to get service principal token for keyvault (used in SP authentication only)
// KeyVaultBaseURL: URL for the keyvault
// KeyVaultSecretName: Name of the keyvault certificate used to obtain the certificate
// AzureEnvironment: Name of azure cloud environment. Possible values: AzurePublicCloud, AzureUSGovernmentCloud, AzureChinaCloud and AzureGermanCloud
type Credentials struct {
	ApplicationID         string `json:"applicationID" env:"APPLICATION_ID,required"`
	TenantID              string `json:"tenantID" env:"AZURE_TENANT_ID,required"`
	SpManagedIdentityID   string `json:"spManagedIdentityID" env:"SP_MANAGED_IDENTITY_ID,required"`
	MsiManagedIdentityID  string `json:"msiManagedIdentityID" env:"MSI_MANAGED_IDENTITY_ID,required"`
	ManagedIdentitySecret string `json:"keyVaultSecret" env:"MANAGED_IDENTITY_SECRET,required"`
	KeyVaultBaseURL       string `json:"keyVaultBaseURL" env:"KEYVAULT_URL,required"`
	KeyVaultSecretName    string `json:"keyVaultSecretName" env:"KVAULT_SECRET_NAME,required"`
	AzureEnvironment      string `json:"azureCloudName" env:"AZURE_ENVIRONMENT,required"`
}

// NewCredentialsFromEnvironment creates a new instance of Credentials from environment variables.
// Output:
//	- Credentials: A new instance of Credentials struct.
// 	- Error, if any.
func NewCredentialsFromEnvironment() (*Credentials, error) {
	credentials := Credentials{}
	err := env.Parse(&credentials)
	if err != nil {
		return nil, err
	}
	return &credentials, nil
}

// ToJSONString converts an instance of Credentials struct to JSON string.
// Receiver:
//	- c : An instance of Credentials
// Output:
//	- JSON string representation of Credentials
//	- Error, if any.
func (c *Credentials) ToJSONString() (string, error) {
	credBytes, err := json.Marshal(*c)
	if err != nil {
		return "", nil
	}

	return string(credBytes), nil
}
