// Copyright (c) 2020-2021 NetApp, Inc. All rights reserved.

package azure

import (
	"fmt"

	"github.com/caarlos0/env"

	"github.com/greenqloud/hyperscaler-network-client/models/common"
)

const (
	//V1APIVersion defines the string literal for API version 1 of DRP.
	V1APIVersion string = "2018-09-01-preview"

	// V2APIVersion defines the string literal for API version 2 of DRP.
	V2APIVersion string = "2020-10-01-preview"

	// V3APIVersion defines the string literal for API version 3 of DRP.
	V3APIVersion string = "2021-06-01-preview"
)

// DRPConfiguration defines the composite struct that consists of all DRP configuration parameters. These include all parameters read as environment variables.
type DRPConfiguration struct {
	ClientConfig      *DrpClientConfig
	AuthConfig        *AuthenticationConfig
	Credentials       *Credentials
	ConfigClusterList common.ClusterList
}

// DrpClientConfig defines the configuration parameters required for DRP client initialization.
type DrpClientConfig struct {
	BaseURI                string `env:"DRP_BASE_URI,required"`
	RegionOverride         string `env:"DRP_REGION_OVERRIDE"`
	Vnet                   string `env:"DRP_VNET"`
	RetryAttempts          int    `env:"DRP_RETRY_ATTEMPTS,required"`
	RetryBackoffDeltaSec   int    `env:"DRP_RETRY_BACKOFF_DELTA_SEC,required"` // Back-off interval between requests.
	RetryBackoffMaxSec     int    `env:"DRP_RETRY_BACKOFF_MAX_SEC,required"`   // Maximum possible delay between requests.
	RequestTimeoutDuration int    `env:"DRP_TIMEOUT_SEC,required"`             // Total time after which request will timeout.
	AuthenticationEnabled  bool   `env:"DRP_ENABLE_AUTHENTICATION,required"`
	APIVersion             string `env:"DRP_API_VERSION,required"`
}

// NewDRPConfiguration creates a new drpConfiguration object based on all DRP related parameters defined in environment.
// Output:
//	- An instance of drpConfiguration object.
//	- Error.
func NewDRPConfiguration() (*DRPConfiguration, error) {
	clientConfig, err := newDRPClientConfigFromEnv()
	if err != nil {
		return nil, fmt.Errorf("error in retrieving DRP client configuration parameters from environment: %v", err)
	}

	if !clientConfig.AuthenticationEnabled {
		return &DRPConfiguration{ClientConfig: clientConfig}, nil
	}

	authConfig, err := newDRPAuthServerConfigFromEnv()
	if err != nil {
		return nil, fmt.Errorf("error in retrieving DRP authentication configuration parameters from environment: %v", err)
	}

	authCredentials, err := NewCredentialsFromEnvironment()
	if err != nil {
		return nil, fmt.Errorf("error while obtaining azure credentials form environment : %v", err)
	}

	return &DRPConfiguration{ClientConfig: clientConfig, AuthConfig: authConfig, Credentials: authCredentials}, nil
}

// newDRPClientConfigFromEnv creates a new DrpClientConfig object based on the values set as environment variables.
// Output:
//	- An instance of DrpClientConfig object.
//	- Error.
func newDRPClientConfigFromEnv() (*DrpClientConfig, error) {
	config := DrpClientConfig{}
	err := env.Parse(&config)
	if err != nil {
		return nil, err
	}

	return &config, nil
}

// newDRPAuthServerConfigFromEnv creates a new AuthenticationConfig object based on the values set as environment variables.
// Output:
//	- An instance of AuthenticationConfig object.
//	- Error.
func newDRPAuthServerConfigFromEnv() (*AuthenticationConfig, error) {
	config := AuthenticationConfig{}
	err := env.Parse(&config)
	if err != nil {
		return nil, err
	}
	return &config, nil
}
