// Copyright (c) 2020-2021 NetApp, Inc. All rights reserved.

package common

import (
	"io/ioutil"

	"github.com/greenqloud/go-lib/log"
	"gopkg.in/yaml.v2"
)

// Tag type defines names additional parameters (other than usage limit objects) in DRP response
type Tag string

// ResourceName type defines names of resources for which usage and limits must be tracked.
type ResourceName string

// HyperscalerType type defines the type of hyperscaler to track.
type HyperscalerType string
type NetworkConnectivity string
type NetworkType string
type NetworkSubType string
type NetworkStorageAvailabilityZoneType string

const (
	// NetworkProximity tag name to define network proximity values.
	// Example, Tier2Alias in case of DRP.
	NetworkProximity Tag = "network_proximity"

	// NetworkConnectivity tag name to define advanced networking connectivity.
	// In case of Azure (DRP) it is network_connectivity=NetworkType:NetworkSubtype
	// i.e rotterdam:T1 or rotterdam:T2 or basic:Default
	NetworkTypeSdnAppliance NetworkType = "sdnAppliance"
	NetworkTypeBasic        NetworkType = "basic"
	NetworkTypeNotSet       NetworkType = "__notSet__"

	NetworkSubtypeDefault NetworkSubType = "Default"
	NetworkSubtypeT1      NetworkSubType = "T1"
	NetworkSubtypeT2      NetworkSubType = "T2"
	NetworkSubtypeNotSet  NetworkSubType = "__notSet__"

	NetworkConnectivityDefault NetworkConnectivity = "basic:Default"
	NetworkConnectivityT1      NetworkConnectivity = "sdnAppliance:T1"
	NetworkConnectivityT2      NetworkConnectivity = "sdnAppliance:T2"
	NetworkConnectivityNotSet  NetworkConnectivity = "__notSet__"

	NetworkStorageAvailabilityZoneTypeNotSet NetworkStorageAvailabilityZoneType = "__notSet__"

	// VRFResourceName resource name to define network VRF
	VRFResourceName ResourceName = "vrf"

	// RouteResourceName resource name to define network routes
	RouteResourceName ResourceName = "route"

	// TenancyResourceName resource name to define cvn tenancy count
	TenancyResourceName ResourceName = "tenancy"

	// AzureCluster defines constant for Azure hyperscaler type.
	AzureCluster HyperscalerType = "azure"

	// GCPCluster defines constant for GCP hyperscaler type.
	GCPCluster HyperscalerType = "gcp"
)

// UsageLimit object model type defines standard usage limit objects
type UsageLimit struct {
	Usage int64
	Limit int64
}

// Network object model for hyperscaler network information
type Network struct {
	Proximity                     *string
	Connectivity                  NetworkConnectivity
	Type                          NetworkType
	SubType                       NetworkSubType
	StorageAvailabilityZone       NetworkStorageAvailabilityZoneType
	StorageAvailabilityZoneDetail NetworkStorageAvailabilityZoneType
}

// UsageLimitMap defines the standard map of usage limit objects
// in any hyperscaler cluster GET response.
type UsageLimitMap map[ResourceName]UsageLimit

// HyperscalerGetResponse model type defines the standard parameters expected
// from a Hyperscaler cluster GET response.
// Note: The standard response is composed of:
//	- Cluster name
//	- Usage Limit Map
//	- Additional Tags
type HyperscalerGetResponse struct {
	Name string
	UsageLimitMap
	TagValues map[Tag]*string
	Network   *Network
}

// GetAllDRPTags gets a list of all tags relevant to DRP hyperscaler clusters.
func GetAllDRPTags() []Tag {
	return []Tag{
		NetworkProximity}
}

// GetAllDRPResourceNames gets a list of names of all relevant DRP resources.
func GetAllDRPResourceNames() []ResourceName {
	return []ResourceName{
		VRFResourceName,
		RouteResourceName,
	}
}

// ClusterList
// Config clusters are loaded from yaml file and stored in ClusterList
type ClusterList struct {
	Clusters []*Cluster
}

// Cluster
// structure to read from Yaml File
type Cluster struct {
	ClusterName           string `yaml:"clusterName"`
	VrfCount              int32  `yaml:"vrfCount"`
	VrfLimit              int32  `yaml:"vrfLimit"`
	RouteCount            int32  `yaml:"routeCount"`
	RouteLimit            int32  `yaml:"routeLimit"`
	AllocatedRoutes       int32  `yaml:"allocatedRoutes"`
	AllocatedRoutesLimit  int32  `yaml:"allocatedRoutesLimit"`
	Tier2Alias            string `yaml:"tier2Alias"`
	SdnApplianceEnabled   bool   `yaml:"sdnApplianceEnabled"`
	SdnAppliancePlacement string `yaml:"sdnAppliancePlacement"`
	AvailabilityZone      string `yaml:"availabilityZone"`
	PhysicalZoneIndex     int32  `yaml:"physicalZoneIndex"`
}

// LoadClusterListYaml load data into struct from yaml file
func LoadClusterListYaml(fileName string) (ClusterList, error) {
	clusterList := ClusterList{}
	contents, err := ioutil.ReadFile(fileName)
	if err != nil {
		log.Errorf("Cannot load yaml file %s, err %v", fileName, err)
		return ClusterList{}, err
	}
	err = yaml.Unmarshal([]byte(contents), &clusterList)
	if err != nil {
		log.Errorf("Cannot Unmarshall yaml file %s, err %v", fileName, err)
		return ClusterList{}, err
	}
	return clusterList, nil
}

// MergeUniqueHyperscalerResponse merge two HyperscalerGetResponse, if the cluster is present in first response
// skip it while merging second response.
func MergeUniqueHyperscalerResponse(first []*HyperscalerGetResponse, second []*HyperscalerGetResponse) []*HyperscalerGetResponse {
	var mergeClusters []*HyperscalerGetResponse
	clusterMap := make(map[string]bool)
	for _, cluster := range first {
		if cluster.Name != "" {
			clusterMap[cluster.Name] = true
		}
		mergeClusters = append(mergeClusters, cluster)
	}
	for _, cluster := range second {
		if !clusterMap[cluster.Name] {
			mergeClusters = append(mergeClusters, cluster)
		}
	}
	return mergeClusters
}
